@@ -17,6 +17,7 @@
 
 // Keep these in sync with other top-level files.
 #![allow(clippy::await_holding_refcell_ref)]
+#![allow(clippy::collapsible_else_if)]
 #![warn(anonymous_parameters, bad_style, missing_docs)]
 #![warn(unused, unused_extern_crates, unused_import_braces, unused_qualifications)]
 #![warn(unsafe_code)]
@@ -26,13 +27,16 @@ use std::cell::RefCell;
 use std::rc::Rc;
 
 // TODO(jmmv): Should narrow the exposed interface by 1.0.0.
+pub mod arrays;
 pub mod console;
 mod editor;
 pub mod exec;
 pub mod gpio;
 pub mod help;
 pub mod numerics;
-pub mod store;
+pub mod program;
+pub mod service;
+pub mod storage;
 pub mod strings;
 #[cfg(feature = "crossterm")]
 pub mod terminal;
@@ -117,6 +121,7 @@ impl MachineBuilder {
     /// Builds the interpreter.
     pub fn build(mut self) -> Result<Machine> {
         let mut machine = Machine::default();
+        arrays::add_all(&mut machine);
         console::add_all(&mut machine, self.get_console()?);
         gpio::add_all(&mut machine, self.get_gpio_pins());
         exec::add_all(&mut machine, self.sleep_fn);
@@ -127,7 +132,7 @@ impl MachineBuilder {
 
     /// Extends the machine with interactive (REPL) features.
     pub fn make_interactive(self) -> InteractiveMachineBuilder {
-        InteractiveMachineBuilder { builder: self, program: None, store: None }
+        InteractiveMachineBuilder::from(self)
     }
 }
 
@@ -135,30 +140,43 @@ impl MachineBuilder {
 ///
 /// This is a superset of a `ScriptingMachineBuilder`.
 ///
-/// Unless otherwise specified, the interpreter is connected to an in-memory store and to a stored
+/// Unless otherwise specified, the interpreter is connected to an in-memory drive and to a stored
 /// program that can be edited interactively.
 pub struct InteractiveMachineBuilder {
     builder: MachineBuilder,
-    program: Option<Rc<RefCell<dyn store::Program>>>,
-    store: Option<Rc<RefCell<dyn store::Store>>>,
+    program: Option<Rc<RefCell<dyn program::Program>>>,
+    storage: Rc<RefCell<storage::Storage>>,
+    service: Option<Rc<RefCell<dyn service::Service>>>,
 }
 
 impl InteractiveMachineBuilder {
+    /// Constructs an interactive machine builder from a non-interactive builder.
+    fn from(builder: MachineBuilder) -> Self {
+        let storage = Rc::from(RefCell::from(storage::Storage::default()));
+        InteractiveMachineBuilder { builder, program: None, storage, service: None }
+    }
+
+    /// Lazily initializes the `storage` field with a default value and returns it.
+    pub fn get_storage(&mut self) -> Rc<RefCell<storage::Storage>> {
+        self.storage.clone()
+    }
+
     /// Overrides the default stored program with the given one.
-    pub fn with_program(mut self, program: Rc<RefCell<dyn store::Program>>) -> Self {
+    pub fn with_program(mut self, program: Rc<RefCell<dyn program::Program>>) -> Self {
         self.program = Some(program);
         self
     }
 
-    /// Overrides the default in-memory store with the given one.
-    pub fn with_store(mut self, store: Rc<RefCell<dyn store::Store>>) -> Self {
-        self.store = Some(store);
+    /// Overrides the default service client with the given one.
+    pub fn with_service(mut self, service: Rc<RefCell<dyn service::Service>>) -> Self {
+        self.service = Some(service);
         self
     }
 
     /// Builds the interpreter.
     pub fn build(mut self) -> Result<Machine> {
         let console = self.builder.get_console()?;
+        let storage = self.get_storage();
         let mut machine = self.builder.build()?;
 
         let program = match self.program {
@@ -166,13 +184,15 @@ impl InteractiveMachineBuilder {
             None => Rc::from(RefCell::from(editor::Editor::default())),
         };
 
-        let store = match self.store {
-            Some(store) => store,
-            None => Rc::from(RefCell::from(store::InMemoryStore::default())),
+        let service = match self.service {
+            Some(service) => service,
+            None => Rc::from(RefCell::from(service::CloudService::default())),
         };
 
         help::add_all(&mut machine, console.clone());
-        store::add_all(&mut machine, program, console, store);
+        program::add_all(&mut machine, program, console.clone(), storage.clone());
+        service::add_all(&mut machine, service, console.clone(), storage.clone());
+        storage::add_all(&mut machine, console, storage);
 
         Ok(machine)
     }
