@@ -15,7 +15,8 @@
 
 //! String functions for EndBASIC.
 
-use endbasic_core::ast::{Value, VarType};
+use endbasic_core::ast::{Expr, Value, VarType};
+use endbasic_core::eval::eval_all;
 use endbasic_core::exec::Machine;
 use endbasic_core::syms::{
     CallError, CallableMetadata, CallableMetadataBuilder, Function, FunctionResult, Symbols,
@@ -23,8 +24,8 @@ use endbasic_core::syms::{
 use std::cmp::min;
 use std::rc::Rc;
 
-/// Category string for all functions provided by this module.
-const CATEGORY: &str = "String manipulation";
+/// Category description for all symbols provided by this module.
+const CATEGORY: &str = "String functions";
 
 /// The `LEFT` function.
 pub struct LeftFunction {
@@ -53,7 +54,8 @@ impl Function for LeftFunction {
         &self.metadata
     }
 
-    fn exec(&self, args: Vec<Value>, _symbols: &mut Symbols) -> FunctionResult {
+    fn exec(&self, args: &[Expr], symbols: &mut Symbols) -> FunctionResult {
+        let args = eval_all(args, symbols)?;
         match args.as_slice() {
             [Value::Text(s), Value::Integer(n)] => {
                 if n < &0 {
@@ -91,7 +93,8 @@ impl Function for LenFunction {
         &self.metadata
     }
 
-    fn exec(&self, args: Vec<Value>, _symbols: &mut Symbols) -> FunctionResult {
+    fn exec(&self, args: &[Expr], symbols: &mut Symbols) -> FunctionResult {
+        let args = eval_all(args, symbols)?;
         match args.as_slice() {
             [Value::Text(s)] => {
                 if s.len() > std::i32::MAX as usize {
@@ -128,7 +131,8 @@ impl Function for LtrimFunction {
         &self.metadata
     }
 
-    fn exec(&self, args: Vec<Value>, _symbols: &mut Symbols) -> FunctionResult {
+    fn exec(&self, args: &[Expr], symbols: &mut Symbols) -> FunctionResult {
+        let args = eval_all(args, symbols)?;
         match args.as_slice() {
             [Value::Text(s)] => Ok(Value::Text(s.trim_start().to_owned())),
             _ => Err(CallError::SyntaxError),
@@ -164,7 +168,8 @@ impl Function for MidFunction {
         &self.metadata
     }
 
-    fn exec(&self, args: Vec<Value>, _symbols: &mut Symbols) -> FunctionResult {
+    fn exec(&self, args: &[Expr], symbols: &mut Symbols) -> FunctionResult {
+        let args = eval_all(args, symbols)?;
         match args.as_slice() {
             [Value::Text(s), Value::Integer(start), Value::Integer(length)] => {
                 if *start < 0 {
@@ -209,7 +214,8 @@ impl Function for RightFunction {
         &self.metadata
     }
 
-    fn exec(&self, args: Vec<Value>, _symbols: &mut Symbols) -> FunctionResult {
+    fn exec(&self, args: &[Expr], symbols: &mut Symbols) -> FunctionResult {
+        let args = eval_all(args, symbols)?;
         match args.as_slice() {
             [Value::Text(s), Value::Integer(n)] => {
                 if n < &0 {
@@ -247,7 +253,8 @@ impl Function for RtrimFunction {
         &self.metadata
     }
 
-    fn exec(&self, args: Vec<Value>, _symbols: &mut Symbols) -> FunctionResult {
+    fn exec(&self, args: &[Expr], symbols: &mut Symbols) -> FunctionResult {
+        let args = eval_all(args, symbols)?;
         match args.as_slice() {
             [Value::Text(s)] => Ok(Value::Text(s.trim_end().to_owned())),
             _ => Err(CallError::SyntaxError),
