@@ -17,7 +17,12 @@
 
 use crate::console::{self, ClearType, Console, Key, Position};
 use crate::gpio;
-use crate::store::{InMemoryStore, Program, Store};
+use crate::program::Program;
+use crate::service::{
+    AccessToken, GetFileRequest, GetFileResponse, GetFilesResponse, LoginRequest, LoginResult,
+    PatchFileRequest, Service,
+};
+use crate::storage::Storage;
 use async_trait::async_trait;
 use endbasic_core::ast::{Value, VarType};
 use endbasic_core::exec::{self, Machine, StopReason};
@@ -73,6 +78,9 @@ pub struct MockConsole {
 
     /// The size of the mock console.
     size: Position,
+
+    /// Whether the console is interactive or not.
+    interactive: bool,
 }
 
 impl Default for MockConsole {
@@ -81,6 +89,7 @@ impl Default for MockConsole {
             golden_in: VecDeque::new(),
             captured_out: vec![],
             size: Position { row: std::usize::MAX, column: std::usize::MAX },
+            interactive: false,
         }
     }
 }
@@ -114,10 +123,14 @@ impl MockConsole {
     pub fn set_size(&mut self, size: Position) {
         self.size = size;
     }
-}
 
-impl Drop for MockConsole {
-    fn drop(&mut self) {
+    /// Sets whether the mock console is interactive or not.
+    pub fn set_interactive(&mut self, interactive: bool) {
+        self.interactive = interactive;
+    }
+
+    /// Ensures that all prerecorded input characters were consumed.
+    fn verify_all_used(&mut self) {
         assert!(
             self.golden_in.is_empty(),
             "Not all golden input chars were consumed; {} left",
@@ -149,7 +162,7 @@ impl Console for MockConsole {
     }
 
     fn is_interactive(&self) -> bool {
-        false
+        self.interactive
     }
 
     fn leave_alt(&mut self) -> io::Result<()> {
@@ -204,7 +217,7 @@ pub struct RecordedProgram {
 #[async_trait(?Send)]
 impl Program for RecordedProgram {
     async fn edit(&mut self, console: &mut dyn Console) -> io::Result<()> {
-        let append = console::read_line(console, "", "").await?;
+        let append = console::read_line(console, "", "", None).await?;
         self.content.push_str(&append);
         self.content.push('\n');
         Ok(())
@@ -219,12 +232,201 @@ impl Program for RecordedProgram {
     }
 }
 
+/// Service client implementation that allows specifying expectations on requests and yields the
+/// responses previously recorded into it.
+#[derive(Default)]
+pub struct MockService {
+    access_token: Option<AccessToken>,
+
+    mock_login: VecDeque<(LoginRequest, io::Result<LoginResult>)>,
+    mock_get_files: VecDeque<(String, io::Result<GetFilesResponse>)>,
+    mock_get_file: VecDeque<((String, String, GetFileRequest), io::Result<GetFileResponse>)>,
+    mock_patch_file: VecDeque<((String, String, PatchFileRequest), io::Result<()>)>,
+    mock_delete_file: VecDeque<((String, String), io::Result<()>)>,
+}
+
+impl MockService {
+    /// The valid username that the mock service recognizes.
+    pub(crate) const USERNAME: &'static str = "mock-username";
+
+    /// The valid password that the mock service recognizes.
+    pub(crate) const PASSWORD: &'static str = "mock-password";
+
+    /// Performs an explicit authentication for those tests that don't go through the `LOGIN`
+    /// command logic.
+    #[cfg(test)]
+    pub(crate) async fn do_authenticate(&mut self) -> AccessToken {
+        self.authenticate(MockService::USERNAME, MockService::PASSWORD).await.unwrap()
+    }
+
+    /// Records the behavior of an upcoming login operation with a request that looks like
+    /// `exp_request` and that returns `result`.
+    #[cfg(test)]
+    pub(crate) fn add_mock_login(
+        &mut self,
+        exp_request: LoginRequest,
+        result: io::Result<LoginResult>,
+    ) {
+        self.mock_login.push_back((exp_request, result));
+    }
+
+    /// Records the behavior of an upcoming "get files" operation for `username` and that returns
+    /// `result`.
+    #[cfg(test)]
+    pub(crate) fn add_mock_get_files(
+        &mut self,
+        username: &str,
+        result: io::Result<GetFilesResponse>,
+    ) {
+        let exp_request = username.to_owned();
+        self.mock_get_files.push_back((exp_request, result));
+    }
+
+    /// Records the behavior of an upcoming "get file" operation for the `username`/`filename`
+    /// pair with a request that looks like `exp_request` and that returns `result`.
+    #[cfg(test)]
+    pub(crate) fn add_mock_get_file(
+        &mut self,
+        username: &str,
+        filename: &str,
+        exp_request: GetFileRequest,
+        result: io::Result<GetFileResponse>,
+    ) {
+        let exp_request = (username.to_owned(), filename.to_owned(), exp_request);
+        self.mock_get_file.push_back((exp_request, result));
+    }
+
+    /// Records the behavior of an upcoming "patch file" operation for the `username`/`filename`
+    /// pair with a request that looks like `exp_request` and that returns `result`.
+    #[cfg(test)]
+    pub(crate) fn add_mock_patch_file(
+        &mut self,
+        username: &str,
+        filename: &str,
+        exp_request: PatchFileRequest,
+        result: io::Result<()>,
+    ) {
+        let exp_request = (username.to_owned(), filename.to_owned(), exp_request);
+        self.mock_patch_file.push_back((exp_request, result));
+    }
+
+    /// Records the behavior of an upcoming "delete file" operation for the `username`/`filename`
+    /// pair and that returns `result`.
+    #[cfg(test)]
+    pub(crate) fn add_mock_delete_file(
+        &mut self,
+        username: &str,
+        filename: &str,
+        result: io::Result<()>,
+    ) {
+        let exp_request = (username.to_owned(), filename.to_owned());
+        self.mock_delete_file.push_back((exp_request, result));
+    }
+
+    /// Ensures that all requests and responses have been consumed.
+    pub(crate) fn verify_all_used(&mut self) {
+        assert!(self.mock_login.is_empty(), "Mock requests not fully consumed");
+        assert!(self.mock_get_files.is_empty(), "Mock requests not fully consumed");
+        assert!(self.mock_get_file.is_empty(), "Mock requests not fully consumed");
+        assert!(self.mock_patch_file.is_empty(), "Mock requests not fully consumed");
+        assert!(self.mock_delete_file.is_empty(), "Mock requests not fully consumed");
+    }
+}
+
+#[async_trait(?Send)]
+impl Service for MockService {
+    async fn authenticate(&mut self, username: &str, password: &str) -> io::Result<AccessToken> {
+        assert!(self.access_token.is_none(), "authenticate called more than once");
+
+        if username != MockService::USERNAME {
+            return Err(io::Error::new(io::ErrorKind::PermissionDenied, "Unknown user"));
+        }
+        if password != MockService::PASSWORD {
+            return Err(io::Error::new(io::ErrorKind::PermissionDenied, "Invalid password"));
+        }
+
+        let access_token = format!("{}", rand::random::<u64>());
+        self.access_token = Some(AccessToken::new(access_token.clone()));
+        Ok(AccessToken::new(access_token))
+    }
+
+    async fn login(
+        &mut self,
+        access_token: &AccessToken,
+        request: &LoginRequest,
+    ) -> io::Result<LoginResult> {
+        assert_eq!(self.access_token.as_ref().expect("authenticate not called yet"), access_token);
+
+        let mock = self.mock_login.pop_front().expect("No mock requests available");
+        assert_eq!(&mock.0, request);
+        mock.1
+    }
+
+    async fn get_files(
+        &mut self,
+        access_token: &AccessToken,
+        username: &str,
+    ) -> io::Result<GetFilesResponse> {
+        assert_eq!(self.access_token.as_ref().expect("authenticate not called yet"), access_token);
+        let mock = self.mock_get_files.pop_front().expect("No mock requests available");
+        assert_eq!(&mock.0, username);
+        mock.1
+    }
+
+    async fn get_file(
+        &mut self,
+        access_token: &AccessToken,
+        username: &str,
+        filename: &str,
+        request: &GetFileRequest,
+    ) -> io::Result<GetFileResponse> {
+        assert_eq!(self.access_token.as_ref().expect("authenticate not called yet"), access_token);
+
+        let mock = self.mock_get_file.pop_front().expect("No mock requests available");
+        assert_eq!(&mock.0 .0, username);
+        assert_eq!(&mock.0 .1, filename);
+        assert_eq!(&mock.0 .2, request);
+        mock.1
+    }
+
+    async fn patch_file(
+        &mut self,
+        access_token: &AccessToken,
+        username: &str,
+        filename: &str,
+        request: &PatchFileRequest,
+    ) -> io::Result<()> {
+        assert_eq!(self.access_token.as_ref().expect("authenticate not called yet"), access_token);
+
+        let mock = self.mock_patch_file.pop_front().expect("No mock requests available");
+        assert_eq!(&mock.0 .0, username);
+        assert_eq!(&mock.0 .1, filename);
+        assert_eq!(&mock.0 .2, request);
+        mock.1
+    }
+
+    async fn delete_file(
+        &mut self,
+        access_token: &AccessToken,
+        username: &str,
+        filename: &str,
+    ) -> io::Result<()> {
+        assert_eq!(self.access_token.as_ref().expect("authenticate not called yet"), access_token);
+
+        let mock = self.mock_delete_file.pop_front().expect("No mock requests available");
+        assert_eq!(&mock.0 .0, username);
+        assert_eq!(&mock.0 .1, filename);
+        mock.1
+    }
+}
+
 /// Builder pattern to prepare an EndBASIC machine for testing purposes.
 #[must_use]
 pub struct Tester {
     console: Rc<RefCell<MockConsole>>,
-    store: Rc<RefCell<InMemoryStore>>,
+    storage: Rc<RefCell<Storage>>,
     program: Rc<RefCell<RecordedProgram>>,
+    service: Rc<RefCell<MockService>>,
     machine: Machine,
 }
 
@@ -232,8 +434,8 @@ impl Default for Tester {
     /// Creates a new tester for a fully-equipped (interactive) machine.
     fn default() -> Self {
         let console = Rc::from(RefCell::from(MockConsole::default()));
-        let store = Rc::from(RefCell::from(InMemoryStore::default()));
         let program = Rc::from(RefCell::from(RecordedProgram::default()));
+        let service = Rc::from(RefCell::from(MockService::default()));
 
         // Default to the pins set that always returns errors.  We could have implemented a set of
         // fake pins here to track GPIO state changes in a nicer way, similar to how we track all
@@ -242,27 +444,34 @@ impl Default for Tester {
         // everywhere instead of having yet another implementation in this module.
         let gpio_pins = Rc::from(RefCell::from(gpio::NoopPins::default()));
 
-        let machine = crate::MachineBuilder::default()
+        let mut builder = crate::MachineBuilder::default()
             .with_console(console.clone())
             .with_gpio_pins(gpio_pins)
             .make_interactive()
-            .with_store(store.clone())
             .with_program(program.clone())
-            .build()
-            .unwrap();
+            .with_service(service.clone());
 
-        Self { console, store, program, machine }
+        // Grab access to the machine's storage subsystem before we lose track of it, as we will
+        // need this to check its state.
+        let storage = builder.get_storage();
+
+        let machine = builder.build().unwrap();
+
+        Self { console, storage, program, service, machine }
     }
 }
 
 impl Tester {
-    /// Creates a new tester using the given `Machine`.
-    pub fn from(machine: Machine) -> Self {
+    /// Creates a new tester with an empty `Machine`.
+    pub fn empty() -> Self {
         let console = Rc::from(RefCell::from(MockConsole::default()));
-        let store = Rc::from(RefCell::from(InMemoryStore::default()));
+        let storage = Rc::from(RefCell::from(Storage::default()));
         let program = Rc::from(RefCell::from(RecordedProgram::default()));
+        let service = Rc::from(RefCell::from(MockService::default()));
 
-        Self { console, store, program, machine }
+        let machine = Machine::default();
+
+        Self { console, storage, program, service, machine }
     }
 
     /// Registers the given builtin command into the machine, which must not yet be registered.
@@ -299,14 +508,6 @@ impl Tester {
         self.console.clone()
     }
 
-    /// Gets the in-memory store from the tester.
-    ///
-    /// This method should generally not be used.  Its primary utility is to hook
-    /// externally-instantiated commands into the testing features.
-    pub fn get_store(&self) -> Rc<RefCell<InMemoryStore>> {
-        self.store.clone()
-    }
-
     /// Gets the recorded program from the tester.
     ///
     /// This method should generally not be used.  Its primary utility is to hook
@@ -315,6 +516,22 @@ impl Tester {
         self.program.clone()
     }
 
+    /// Gets the storage subsystem from the tester.
+    ///
+    /// This method should generally not be used.  Its primary utility is to hook
+    /// externally-instantiated commands into the testing features.
+    pub fn get_storage(&self) -> Rc<RefCell<Storage>> {
+        self.storage.clone()
+    }
+
+    /// Gets the mock service client from the tester.
+    ///
+    /// This method should generally not be used.  Its primary utility is to hook
+    /// externally-instantiated commands into the testing features.
+    pub fn get_service(&self) -> Rc<RefCell<MockService>> {
+        self.service.clone()
+    }
+
     /// Sets the initial contents of the recorded program to `text`.  Can only be called once and
     /// `text` must not be empty.
     pub fn set_program(self, text: &str) -> Self {
@@ -327,9 +544,9 @@ impl Tester {
         self
     }
 
-    /// Creates or overwrites a file in the store.
+    /// Creates or overwrites a file in the storage medium.
     pub fn write_file(self, name: &str, content: &str) -> Self {
-        self.store.borrow_mut().put(name, content).unwrap();
+        block_on(self.storage.borrow_mut().put(name, content)).unwrap();
         self
     }
 
@@ -348,7 +565,7 @@ pub struct Checker<'a> {
     result: exec::Result<StopReason>,
     exp_result: Result<StopReason, String>,
     exp_output: Vec<CapturedOut>,
-    exp_store: HashMap<String, String>,
+    exp_drives: HashMap<String, String>,
     exp_program: String,
     exp_arrays: HashMap<String, Array>,
     exp_vars: HashMap<String, Value>,
@@ -365,7 +582,7 @@ impl<'a> Checker<'a> {
             result,
             exp_result: Ok(StopReason::Eof),
             exp_output: vec![],
-            exp_store: HashMap::default(),
+            exp_drives: HashMap::default(),
             exp_program: String::new(),
             exp_arrays: HashMap::default(),
             exp_vars: HashMap::default(),
@@ -431,11 +648,13 @@ impl<'a> Checker<'a> {
         self
     }
 
-    /// Adds a file to expect in the store with a `name` and specific `content`.
+    /// Adds a file to expect in the drive with a `name` and specific `content`.
+    ///
+    /// `name` must be the absolute path to the file that is expected, including the drive name.
     pub fn expect_file<N: Into<String>, C: Into<String>>(mut self, name: N, content: C) -> Self {
         let name = name.into();
-        assert!(!self.exp_store.contains_key(&name));
-        self.exp_store.insert(name, content.into());
+        assert!(!self.exp_drives.contains_key(&name));
+        self.exp_drives.insert(name, content.into());
         self
     }
 
@@ -500,11 +719,34 @@ impl<'a> Checker<'a> {
             }
         }
 
+        let drive_contents = {
+            let mut files = HashMap::new();
+            let storage = self.tester.storage.borrow();
+            for (drive_name, target) in storage.mounted().iter() {
+                if target.starts_with("cloud://") {
+                    // TODO(jmmv): Verifying the cloud drives is hard because we would need to mock
+                    // out the requests issued by the checks below.  Ignore them for now.
+                    continue;
+                }
+
+                let root = format!("{}:/", drive_name);
+                for name in block_on(storage.enumerate(&root)).unwrap().dirents().keys() {
+                    let path = format!("{}{}", root, name);
+                    let content = block_on(storage.get(&path)).unwrap();
+                    files.insert(path, content);
+                }
+            }
+            files
+        };
+
         assert_eq!(self.exp_vars, vars);
         assert_eq!(self.exp_arrays, arrays);
         assert_eq!(self.exp_output, self.tester.console.borrow().captured_out());
         assert_eq!(self.exp_program, self.tester.program.borrow().text());
-        assert_eq!(self.exp_store, *self.tester.store.borrow().as_hashmap());
+        assert_eq!(self.exp_drives, drive_contents);
+
+        self.tester.console.borrow_mut().verify_all_used();
+        self.tester.service.borrow_mut().verify_all_used();
     }
 }
 
